@@ -41,6 +41,10 @@ class databaseTable(database):
     
     @beartype
     def create_table(self, keys:list, datatype:list):
+        """
+        keys: column names ["name1", "name2", ...]
+        datatype: type of columns ["text", "float", ...]
+        """
         try:
             conn = psycopg2.connect(host=self.hostName,database=self.databaseName,user=self.userName, password=self.sqlPasswords)
             cur = conn.cursor()
@@ -60,6 +64,9 @@ class databaseTable(database):
     
     @beartype
     def insert_one_row(self, insert_data:list, with_id:Optional[bool]=False):
+        """
+        if with_id = True, 'id' should be the last element in 'insert_data'
+        """
         try:
             conn = psycopg2.connect(host=self.hostName,database=self.databaseName,user=self.userName, password=self.sqlPasswords)
             cur  = conn.cursor()        
@@ -83,7 +90,43 @@ class databaseTable(database):
         except psycopg2.Error as err:
             return err
     
+    @beartype
+    def insert_multiple_rows(self, insert_df:pd.DataFrame): 
+        """
+        insert_data: list of lists, e.g.: [[1,2,3], [1,2,3]]
+        """
+        try:
+            conn = psycopg2.connect(host=self.hostName,database=self.databaseName,user=self.userName, password=self.sqlPasswords)
+            cur  = conn.cursor()        
+            sql = "SELECT * FROM {};".format(self.tableName)
+            cur.execute(sql)
+            keys = [i[0] for i in cur.description][0:-2]
+            # insert data
+            qs = ",".join(["%s" for _ in range(len(keys)+2)])
+            insert = """INSERT INTO {} ({},id,date) 
+                    VALUES ({})""".format(self.tableName, ",".join(keys), qs)
+            
+            id                 = secrets.token_hex(16)
+            date               = str(datetime.now())[0:19]
+            len_of_df          = len(insert_df)
+            insert_df["id"]    = [id for i in range(len_of_df)]
+            insert_df["date"]  = [date for i in range(len_of_df)]
+
+            sql_insert_data = []
+            for i in range(len(insert_df)):
+                sql_insert_data.append(tuple(insert_df.iloc[i].to_list()))
+
+            cur.executemany(insert, sql_insert_data)
+            conn.commit()
+            conn.close()
+            return True
+        except psycopg2.Error as err:
+            return err
+    
     def get_all(self):
+        """
+        get all rows in table
+        """
         try:
             conn = psycopg2.connect(host=self.hostName,database=self.databaseName,user=self.userName, password=self.sqlPasswords)
             cur = conn.cursor()
@@ -101,6 +144,9 @@ class databaseTable(database):
     
     @beartype
     def get_one_row(self, id:str):
+        """
+        get one row based on id
+        """
         try:
             conn = psycopg2.connect(host=self.hostName,database=self.databaseName,user=self.userName, password=self.sqlPasswords)
             cur = conn.cursor()
@@ -116,6 +162,10 @@ class databaseTable(database):
 
     @beartype
     def get_columns(self, columnNames:list):
+        """
+        get all columns based on the given column names,
+        columnNames = ["name1", "name2", ...]
+        """
         if columnNames == []:
             return "No column specified"
         try:
@@ -126,6 +176,9 @@ class databaseTable(database):
     
     @beartype
     def delete_one_row(self, id:str):
+        """
+        delete one row from table
+        """
         try:
             conn = psycopg2.connect(host=self.hostName,database=self.databaseName,user=self.userName, password=self.sqlPasswords)
             cur = conn.cursor()
@@ -139,6 +192,9 @@ class databaseTable(database):
     
     @beartype
     def update_one_row(self, id:str, update_data:list):
+        """
+        update the whole row in table
+        """
         try:
             df = self.get_columns(["id"])
             if id in df["id"].to_list():
@@ -153,7 +209,40 @@ class databaseTable(database):
         except psycopg2.Error as err:
             return err
     
+    @beartype
+    def update_elements(self, id:str, update_dict=dict):
+        """
+        update_dict = {"text": text, "list": [1,2,3] }
+        """
+        keys    = tuple(update_dict.keys())
+        values  = tuple(update_dict.values()) 
+        try:
+            # get all ids in table
+            df = self.get_columns(["id"])
+            if id not in df["id"].to_list():
+                return "id does not exist..."
+            if len(values) > 1:
+                keys_s  = f"{keys}".replace("'", '')
+                sql = f"""UPDATE {self.tableName} SET {keys_s} = {values} WHERE id = '{id}';"""
+            else:
+
+                sql = f"""UPDATE {self.tableName} SET {keys[0]} = '{values[0]}' WHERE id = '{id}';"""
+
+            conn = psycopg2.connect(host=self.hostName,database=self.databaseName,user=self.userName, password=self.sqlPasswords)
+            cur = conn.cursor()
+            
+            cur.execute(sql)
+            conn.commit()
+            conn.close()
+            return True
+        except psycopg2.Error as err:
+            return err
+        
+
     def delete_all_rows(self):
+        """
+        delete all rows from table
+        """
         try:
             conn = psycopg2.connect(host=self.hostName,database=self.databaseName,user=self.userName, password=self.sqlPasswords)
             cur = conn.cursor()
